@@ -2,15 +2,15 @@
 
 #include "kiwixapp.h"
 #include "static_content.h"
-#include <kiwix/tools/networkTools.h>
-#include <kiwix/tools/otherTools.h>
 #include <kiwix/manager.h>
+#include <kiwix/tools.h>
 
 #include <QDebug>
 #include <QUrlQuery>
 #include <QUrl>
 #include <QDir>
 #include <QStorageInfo>
+#include <QMessageBox>
 
 ContentManager::ContentManager(Library* library, kiwix::Downloader* downloader, QObject *parent)
     : QObject(parent),
@@ -51,7 +51,7 @@ QStringList ContentManager::getTranslations(const QStringList &keys)
 QStringList ContentManager::getBookInfos(QString id, const QStringList &keys)
 {
     QStringList values;
-    kiwix::Book* b = [=]()->kiwix::Book* {
+    const kiwix::Book* b = [=]()->const kiwix::Book* {
         try {
             return &mp_library->getBookById(id);
         } catch (...) {
@@ -109,7 +109,6 @@ QStringList ContentManager::getBookInfos(QString id, const QStringList &keys)
             if (displayTagMap["_videos"]) displayTagList << tr("Videos");
             if (displayTagMap["_pictures"]) displayTagList << tr("Pictures");
             if (!displayTagMap["_details"]) displayTagList << tr("Introduction only");
-            if (displayTagMap["_ftindex"]) displayTagList << tr("Fulltext index");
             QString s = displayTagList.join(", ");
             values.append(s);
         }
@@ -121,7 +120,20 @@ QStringList ContentManager::getBookInfos(QString id, const QStringList &keys)
 void ContentManager::openBook(const QString &id)
 {
     QUrl url("zim://"+id+".zim/");
-    KiwixApp::instance()->openUrl(url, true);
+    try {
+        KiwixApp::instance()->openUrl(url, true);
+    } catch (const std::exception& e) {
+        auto tabBar = KiwixApp::instance()->getTabWidget();
+        tabBar->closeTab(1);
+        auto text = gt("zim-open-fail-text");
+        text = text.replace("{{ZIM}}", QString::fromStdString(mp_library->getBookById(id).getPath()));
+        auto title = gt("zim-open-fail-title");
+        KiwixApp::instance()->showMessage(text, title, QMessageBox::Warning);
+        mp_library->removeBookFromLibraryById(id);
+        tabBar->setCurrentIndex(0);
+        emit(booksChanged());
+        return;
+    }
 }
 
 #define ADD_V(KEY, METH) {if(key==KEY) {values.append(QString::fromStdString((d->METH()))); continue;}}
@@ -138,7 +150,9 @@ QStringList ContentManager::updateDownloadInfos(QString id, const QStringList &k
     try {
         d = mp_downloader->getDownload(b.getDownloadId());
     } catch(...) {
-        b.setDownloadId("");
+        kiwix::Book bCopy(b);
+        bCopy.setDownloadId("");
+        mp_library->getKiwixLibrary().addOrUpdateBook(bCopy);
         mp_library->save();
         emit(mp_library->booksChanged());
         return values;
@@ -147,9 +161,13 @@ QStringList ContentManager::updateDownloadInfos(QString id, const QStringList &k
     d->updateStatus(true);
     if (d->getStatus() == kiwix::Download::K_COMPLETE) {
         QString tmp(QString::fromStdString(d->getPath()));
-        b.setPath(QDir::toNativeSeparators(tmp).toStdString());
-        b.setDownloadId("");
-        b.setPathValid(true);
+        kiwix::Book bCopy(b);
+        bCopy.setPath(QDir::toNativeSeparators(tmp).toStdString());
+        bCopy.setDownloadId("");
+        bCopy.setPathValid(true);
+        // removing book url so that download link in kiwix-serve is not displayed.
+        bCopy.setUrl("");
+        mp_library->getKiwixLibrary().addOrUpdateBook(bCopy);
         mp_library->save();
         mp_library->bookmarksChanged();
         if (!m_local) {
@@ -208,14 +226,15 @@ QString ContentManager::downloadBook(const QString &id)
 {
     if (!mp_downloader)
         return "";
-    auto& book = [&]()->kiwix::Book& {
+    const auto& book = [&]()->const kiwix::Book& {
         try {
             return m_remoteLibrary.getBookById(id.toStdString());
         } catch (...) {
             return mp_library->getBookById(id);
         }
     }();
-    QStorageInfo storage(QString::fromStdString(getDataDirectory()));
+    auto downloadPath = KiwixApp::instance()->getSettingsManager()->getDownloadDir();
+    QStorageInfo storage(downloadPath);
     if (book.getSize() > storage.bytesAvailable()) {
         return "storage_error";
     }
@@ -225,15 +244,15 @@ QString ContentManager::downloadBook(const QString &id)
             return "";
     kiwix::Download *download;
     try {
-        auto downloadPath = KiwixApp::instance()->getSettingsManager()->getDownloadDir();
         std::pair<std::string, std::string> downloadDir("dir", downloadPath.toStdString());
         const std::vector<std::pair<std::string, std::string>> options = { downloadDir };
         download = mp_downloader->startDownload(book.getUrl(), options);
     } catch (std::exception& e) {
         return "";
     }
-    book.setDownloadId(download->getDid());
-    mp_library->addBookToLibrary(book);
+    kiwix::Book bookCopy(book);
+    bookCopy.setDownloadId(download->getDid());
+    mp_library->addBookToLibrary(bookCopy);
     mp_library->save();
     emit(oneBookChanged(id));
     return QString::fromStdString(download->getDid());
@@ -250,18 +269,10 @@ void ContentManager::eraseBookFilesFromComputer(const QString dirPath, const QSt
 void ContentManager::eraseBook(const QString& id)
 {
     auto tabBar = KiwixApp::instance()->getTabWidget();
-    int i = 1;
-    while (i < tabBar->count() - 1) {
-        WebView* webView = tabBar->widget(i)->getWebView();
-        if (webView->zimId() == id) {
-            tabBar->closeTab(i);
-        } else {
-            i++;
-        }
-    }
+    tabBar->closeTabsByZimId(id);
     kiwix::Book book = mp_library->getBookById(id);
-    QString dirPath = QString::fromStdString(removeLastPathElement(book.getPath()));
-    QString filename = QString::fromStdString(getLastPathElement(book.getPath())) + "*";
+    QString dirPath = QString::fromStdString(kiwix::removeLastPathElement(book.getPath()));
+    QString filename = QString::fromStdString(kiwix::getLastPathElement(book.getPath())) + "*";
     eraseBookFilesFromComputer(dirPath, filename);
     mp_library->removeBookFromLibraryById(id);
     mp_library->save();
@@ -306,8 +317,8 @@ void ContentManager::cancelBook(const QString& id)
     if (download->getStatus() != kiwix::Download::K_COMPLETE) {
         download->cancelDownload();
     }
-    QString dirPath = QString::fromStdString(removeLastPathElement(download->getPath()));
-    QString filename = QString::fromStdString(getLastPathElement(download->getPath())) + "*";
+    QString dirPath = QString::fromStdString(kiwix::removeLastPathElement(download->getPath()));
+    QString filename = QString::fromStdString(kiwix::getLastPathElement(download->getPath())) + "*";
     eraseBookFilesFromComputer(dirPath, filename);
     mp_library->removeBookFromLibraryById(id);
     mp_library->save();
@@ -345,6 +356,12 @@ void ContentManager::setCurrentCategoryFilter(QString category)
     emit(filterParamsChanged());
 }
 
+void ContentManager::setCurrentContentTypeFilter(QList<ContentTypeFilter*>& contentTypeFilters)
+{
+    m_contentTypeFilters = contentTypeFilters;
+    emit(filterParamsChanged());
+}
+
 void ContentManager::updateLibrary() {
     if (m_local) {
         emit(pendingRequest(false));
@@ -375,20 +392,33 @@ void ContentManager::setSearch(const QString &search)
 QStringList ContentManager::getBookIds()
 {
     kiwix::Filter filter;
-    std::vector<std::string> tags;
+    std::vector<std::string> acceptTags, rejectTags;
     if (m_categoryFilter != "all" && m_categoryFilter != "other") {
-        tags.push_back("_category:"+m_categoryFilter.toStdString());
-        filter.acceptTags(tags);
+        acceptTags.push_back("_category:"+m_categoryFilter.toStdString());
     }
     if (m_categoryFilter == "other") {
         for (auto& category: S_CATEGORIES) {
             if (category.first != "other" && category.first != "all") {
-                tags.push_back("_category:"+category.first.toStdString());
+                rejectTags.push_back("_category:"+category.first.toStdString());
             }
         }
-        filter.rejectTags(tags);
     }
+
+    for (auto &contentTypeFilter : m_contentTypeFilters) {
+        auto state = contentTypeFilter->checkState();
+        auto filter = contentTypeFilter->getName();
+        if (state == Qt::PartiallyChecked) {
+            acceptTags.push_back("_" + filter.toStdString() +":yes");
+        } else if (state == Qt::Checked) {
+            acceptTags.push_back("_" + filter.toStdString() +":no");
+        }
+    }
+
+    filter.acceptTags(acceptTags);
+    filter.rejectTags(rejectTags);
     filter.query(m_searchQuery.toStdString());
+    if (m_currentLanguage != "*")
+        filter.lang(m_currentLanguage.toStdString());
 
     if (m_local) {
         filter.local(true);
