@@ -1,6 +1,9 @@
 #include "kiwixapp.h"
 #include "static_content.h"
 #include "zim/error.h"
+#include "zim/version.h"
+#include "kiwix/tools.h"
+#include "kiwix/version.h"
 
 #include <QLocale>
 #include <QLibraryInfo>
@@ -13,19 +16,24 @@
 #include <QPrintDialog>
 #include <thread>
 #include <QMessageBox>
+#ifdef Q_OS_WIN
+#include <QtPlatformHeaders\QWindowsWindowFunctions>
+#endif
+
+////////////////////////////////////////////////////////////////////////////////
+// KiwixApp
+////////////////////////////////////////////////////////////////////////////////
 
 KiwixApp::KiwixApp(int& argc, char *argv[])
     : QtSingleApplication("kiwix-desktop", argc, argv),
-      m_settingsManager(),
       m_profile(),
       m_libraryDirectory(findLibraryDirectory()),
-      m_library(),
+      m_library(m_libraryDirectory),
       mp_downloader(nullptr),
       mp_manager(nullptr),
       mp_mainWindow(nullptr),
-      mp_server(new kiwix::KiwixServe(
-        appendToDirectory(m_libraryDirectory.toStdString(),"library.xml"),
-        m_settingsManager.getKiwixServerPort()))
+      m_nameMapper(m_library.getKiwixLibrary(), false),
+      m_server(&m_library.getKiwixLibrary(), &m_nameMapper)
 {
     try {
         m_translation.setTranslation(QLocale());
@@ -33,8 +41,6 @@ KiwixApp::KiwixApp(int& argc, char *argv[])
         QMessageBox::critical(nullptr, "Translation error", e.what());
         return;
     }
-    qInfo() << "Compiled with Qt Version " << QT_VERSION_STR;
-    qInfo() << "Runtime Qt Version " << qVersion();
     m_qtTranslator.load(QLocale(), "qt", "_",
                         QLibraryInfo::location(QLibraryInfo::TranslationsPath));
     installTranslator(&m_qtTranslator);
@@ -69,39 +75,46 @@ void KiwixApp::init()
     setApplicationName("Kiwix");
     setDesktopFileName("kiwix.desktop");
 
-    setStyle(QStyleFactory::create("Windows"));
     QFile styleFile(":/css/style.css");
     styleFile.open(QIODevice::ReadOnly);
     auto byteContent = styleFile.readAll();
-    styleFile.close();
     QString style(byteContent);
     setStyleSheet(style);
 
 
     createAction();
     mp_mainWindow = new MainWindow;
-    mp_tabWidget = mp_mainWindow->getTabBar();
-    mp_tabWidget->setContentManagerView(mp_manager->getView());
-    mp_tabWidget->setNewTabButton();
-    mp_mainWindow->getSideContentManager()->setContentManager(mp_manager);
-    setSideBar(CONTENTMANAGER_BAR);
+    getTabWidget()->setContentManagerView(mp_manager->getView());
+    getTabWidget()->setNewTabButton();
     postInit();
     mp_errorDialog = new QErrorMessage(mp_mainWindow);
     setActivationWindow(mp_mainWindow);
     mp_mainWindow->show();
+#ifdef Q_OS_WIN
+    QWindow *window = mp_mainWindow->windowHandle();
+    if (window) {
+        QWindowsWindowFunctions::setHasBorderInFullScreen(window, true);
+    }
+#endif
     connect(this, &QtSingleApplication::messageReceived, this, [=](const QString &message) {
         if (!message.isEmpty()) {
             this->openZimFile(message);
         }
     });
+    connect(&m_watcher, &QFileSystemWatcher::directoryChanged, this, [=](QString monitorDir) {
+        m_library.asyncLoadMonitorDir(monitorDir);
+    });
+    QString monitorDir = m_settingsManager.getMonitorDir();
+    if (monitorDir != "") {
+        m_library.setMonitorDirZims(m_library.getLibraryZimsFromDir(monitorDir));
+        m_watcher.addPath(monitorDir);
+        m_library.asyncLoadMonitorDir(monitorDir);
+    }
 }
 
 KiwixApp::~KiwixApp()
 {
-    if (mp_server) {
-        mp_server->shutDown();
-        delete mp_server;
-    }
+    m_server.stop();
     if (mp_downloader) {
         mp_downloader->close();
         delete mp_downloader;
@@ -117,13 +130,13 @@ KiwixApp::~KiwixApp()
 QString KiwixApp::findLibraryDirectory()
 {
   // Check for library.xml in the same directory than the executable (portable kiwix-desktop)
-  auto currentDataDir = QString::fromStdString(removeLastPathElement(getExecutablePath()));
+  auto currentDataDir = QString::fromStdString(kiwix::removeLastPathElement(kiwix::getExecutablePath()));
   auto libraryFile = QFileInfo(currentDataDir, "library.xml");
   if (libraryFile.exists())
     return currentDataDir;
 
   // Check for default dataDirectory.
-  currentDataDir = QString::fromStdString(getDataDirectory());
+  currentDataDir = QString::fromStdString(kiwix::getDataDirectory());
   libraryFile = QFileInfo(currentDataDir, "library.xml");
   if (libraryFile.exists())
     return currentDataDir;
@@ -161,23 +174,28 @@ QString gt(const QString &key) {
 
 void KiwixApp::openZimFile(const QString &zimfile)
 {
-    QString _zimfile = zimfile;
-    if (_zimfile.isEmpty()) {
+    QString _zimfile;
+    if (zimfile.isEmpty()) {
         _zimfile = QFileDialog::getOpenFileName(
-            getMainWindow(),
-            gt("open-zim"),
-            QString(),
-            "ZimFile (*.zim*)");
+                    getMainWindow(),
+                    gt("open-zim"),
+                    QString(),
+                    "ZimFile (*.zim*)");
+
+        if (_zimfile.isEmpty()) {
+            return;
+        }
         _zimfile = QDir::toNativeSeparators(_zimfile);
     }
-    if (_zimfile.isEmpty()) {
-        return;
-    }
+
     QString zimId;
+    const auto &validZimFile = zimfile.isEmpty() ? _zimfile : zimfile;
     try {
-        zimId = m_library.openBookFromPath(_zimfile);
+        zimId = m_library.openBookFromPath(validZimFile);
     } catch (const std::exception& e) {
-        showMessage("Cannot open " + _zimfile + ": \n" + e.what());
+        auto text = gt("zim-open-fail-text");
+        text = text.replace("{{ZIM}}", validZimFile);
+        showMessage(text, gt("zim-open-fail-title"), QMessageBox::Warning);
         return;
     }
     openUrl(QUrl("zim://"+zimId+".zim/"));
@@ -185,19 +203,19 @@ void KiwixApp::openZimFile(const QString &zimfile)
 
 void KiwixApp::printPage()
 {
-    if(!mp_tabWidget->currentWidget())
+    if(!getTabWidget()->currentZimView())
         return;
     QPrinter* printer = new QPrinter();
     QPrintDialog printDialog(printer, mp_mainWindow);
     printDialog.setStyle(nullptr);
     printDialog.setStyleSheet("");
     if (printDialog.exec() == QDialog::Accepted) {
-        auto webview = mp_tabWidget->currentWebView();
+        auto webview = getTabWidget()->currentWebView();
         if(!webview)
             return;
         webview->page()->print(printer, [=](bool success) {
             if (!success) {
-                showMessage("An error has occured while printing.");
+                showMessage(gt("print-page-error"), gt("error-title"), QMessageBox::Critical);
             }
             delete printer;
         });
@@ -209,57 +227,38 @@ void KiwixApp::openUrl(const QString &url, bool newTab) {
 }
 
 void KiwixApp::openUrl(const QUrl &url, bool newTab) {
-    mp_tabWidget->openUrl(url, newTab);
-}
-
-void KiwixApp::setSideBar(KiwixApp::SideBarType type)
-{
-    auto sideDockWidget = mp_mainWindow->getSideDockWidget();
-    switch(type) {
-        case CONTENTMANAGER_BAR:
-        case READINGLIST_BAR:
-            sideDockWidget->setCurrentIndex(type);
-            sideDockWidget->show();
-            break;
-        case NONE:
-            sideDockWidget->hide();
-            break;
-    }
-    m_currentSideType = type;
-    emit(currentSideTypeChanged(type));
-}
-
-void KiwixApp::toggleSideBar(KiwixApp::SideBarType type) {
-    if (m_currentSideType == type) {
-        setSideBar(NONE);
-        return;
-    }
-
-    if (m_currentSideType == CONTENTMANAGER_BAR) {
-        return;
-    }
-    setSideBar(type);
+    getTabWidget()->openUrl(url, newTab);
 }
 
 void KiwixApp::openRandomUrl(bool newTab)
 {
-    auto zimId = mp_tabWidget->currentZimId();
+    auto zimId = getTabWidget()->currentZimId();
     if (zimId.isEmpty()) {
         return;
     }
     auto reader = m_library.getReader(zimId);
-    auto entry = reader->getRandomPage();
+    try {
+        auto entry = reader->getRandomPage();
 
-    QUrl url;
-    url.setScheme("zim");
-    url.setHost(zimId + ".zim");
-    url.setPath("/" + QString::fromStdString(entry.getPath()));
-    openUrl(url, newTab);
+        QUrl url;
+        url.setScheme("zim");
+        url.setHost(zimId + ".zim");
+        url.setPath("/" + QString::fromStdString(entry.getPath()));
+        openUrl(url, newTab);
+    } catch ( const kiwix::NoEntry& ) {
+        showMessage(gt("random-article-error"), gt("error-title"), QMessageBox::Information);
+    }
 }
 
-void KiwixApp::showMessage(const QString &message)
+void KiwixApp::showMessage(const QString &message, const QString &title, const enum QMessageBox::Icon &icon)
 {
-    mp_errorDialog->showMessage(message);
+    QMessageBox msgBox(
+        icon, //Icon
+        title, //Title
+        message, //Text
+        QMessageBox::Ok //Buttons
+    );
+    msgBox.exec();
 }
 
 QAction *KiwixApp::getAction(KiwixApp::Actions action)
@@ -283,49 +282,86 @@ bool KiwixApp::isCurrentArticleBookmarked()
     return false;
 }
 
-#define CREATE_ACTION_ICON(ID, ICON, TEXT) \
-    mpa_actions[ID] = new QAction(QIcon(":/icons/" ICON ".svg"), TEXT)
+void KiwixApp::setMonitorDir(const QString &dir) {
+    m_settingsManager.setMonitorDir(dir);
+    m_library.setMonitorDirZims(QStringList());
+    for (auto path : m_watcher.directories()) {
+        m_watcher.removePath(path);
+    }
+    if (dir != "") {
+        m_watcher.addPath(dir);
+        m_library.asyncLoadMonitorDir(dir);
+    }
+}
+
 #define CREATE_ACTION(ID, TEXT) \
     mpa_actions[ID] = new QAction(TEXT)
-#define SET_SHORTCUT(ID, SHORTCUT) mpa_actions[ID]->setShortcut(SHORTCUT)
+#define SET_SHORTCUT(ID, TEXT, SHORTCUT) \
+    mpa_actions[ID]->setShortcut(SHORTCUT); \
+    mpa_actions[ID]->setToolTip(TEXT + " (" + QKeySequence(SHORTCUT).toString() + ")" )
+#define CREATE_ACTION_SHORTCUT(ID, TEXT, SHORTCUT) \
+    CREATE_ACTION(ID, TEXT); \
+    SET_SHORTCUT(ID, TEXT, SHORTCUT)
+#define CREATE_ACTION_ICON_SHORTCUT(ID, ICON, TEXT, SHORTCUT) \
+    mpa_actions[ID] = new QAction(QIcon(":/icons/" ICON ".svg"), TEXT); \
+    SET_SHORTCUT(ID, TEXT, SHORTCUT)
+#define CREATE_ACTION_ONOFF_ICON_SHORTCUT(ID, ON_ICON, OFF_ICON, TEXT, SHORTCUT) \
+    CREATE_ACTION(ID, TEXT); \
+    SET_SHORTCUT(ID, TEXT, SHORTCUT); \
+    mpa_actions[ID]->setCheckable(true); \
+    {   QIcon icon; \
+        icon.addPixmap(QPixmap(":/icons/" ON_ICON ".svg"), QIcon::Normal, QIcon::On); \
+        icon.addPixmap(QPixmap(":/icons/" OFF_ICON, ".svg"), QIcon::Normal, QIcon::Off); \
+        mpa_actions[ID]->setIcon(icon); }
+#define SET_SHORTCUTS(ID, TEXT, SHORTCUTS) \
+    mpa_actions[ID]->setShortcuts(SHORTCUTS); \
+    mpa_actions[ID]->setToolTip(TEXT + " (" + SHORTCUTS.first().toString() + ")" )
+#define CREATE_ACTION_SHORTCUTS(ID, TEXT, SHORTCUTS) \
+    CREATE_ACTION(ID, TEXT); \
+    SET_SHORTCUTS(ID, TEXT, SHORTCUTS)
 #define HIDE_ACTION(ID) mpa_actions[ID]->setVisible(false)
 #define DISABLE_ACTION(ID) mpa_actions[ID]->setDisabled(true)
 
 void KiwixApp::createAction()
 {
-    CREATE_ACTION_ICON(KiwixServeAction, "share", gt("local-kiwix-server"));
-    SET_SHORTCUT(KiwixServeAction, QKeySequence(Qt::CTRL+Qt::Key_I));
+    CREATE_ACTION_ICON_SHORTCUT(KiwixServeAction, "share", gt("local-kiwix-server"), QKeySequence(Qt::CTRL+Qt::Key_I));
 
-    CREATE_ACTION_ICON(RandomArticleAction, "random", gt("random-article"));
-    SET_SHORTCUT(RandomArticleAction, QKeySequence(Qt::CTRL+Qt::Key_R));
+    CREATE_ACTION_ICON_SHORTCUT(RandomArticleAction, "random", gt("random-article"), QKeySequence(Qt::CTRL+Qt::Key_R));
     connect(mpa_actions[RandomArticleAction], &QAction::triggered,
             this, [=]() { this->openRandomUrl(false); });
 
-    CREATE_ACTION(OpenHomePageAction, gt("random-article"));
-    SET_SHORTCUT(OpenHomePageAction, QKeySequence(Qt::ALT + Qt::Key_Home));
+    CREATE_ACTION_SHORTCUT(OpenHomePageAction, gt("home-page"), QKeySequence(Qt::ALT + Qt::Key_Home));
 
-    CREATE_ACTION_ICON(PrintAction, "print", gt("print"));
-    SET_SHORTCUT(PrintAction, QKeySequence::Print);
+    if (QGuiApplication::isLeftToRight()) {
+      CREATE_ACTION_ICON_SHORTCUT(HistoryBackAction, "history-left", gt("back"), QKeySequence(Qt::ALT + Qt::Key_Left));
+    } else {
+      CREATE_ACTION_ICON_SHORTCUT(HistoryBackAction, "history-right", gt("back"), QKeySequence(Qt::ALT + Qt::Key_Right));
+    }
+    DISABLE_ACTION(HistoryBackAction);
+
+    if (QGuiApplication::isLeftToRight()) {
+      CREATE_ACTION_ICON_SHORTCUT(HistoryForwardAction, "history-right", gt("forward"), QKeySequence(Qt::ALT + Qt::Key_Right));
+    } else {
+      CREATE_ACTION_ICON_SHORTCUT(HistoryForwardAction, "history-left", gt("forward"), QKeySequence(Qt::ALT + Qt::Key_Left));
+    }
+    DISABLE_ACTION(HistoryForwardAction);
+
+    CREATE_ACTION_ICON_SHORTCUT(PrintAction, "print", gt("print"), QKeySequence::Print);
     connect(mpa_actions[PrintAction], &QAction::triggered,
             this, &KiwixApp::printPage);
 
-    CREATE_ACTION(NewTabAction, gt("new-tab"));
-    SET_SHORTCUT(NewTabAction, QKeySequence::AddTab);
+    CREATE_ACTION_ICON_SHORTCUT(NewTabAction,"new-tab-icon", gt("new-tab"), QKeySequence::AddTab);
 
-    CREATE_ACTION_ICON(CloseTabAction, "close", gt("close-tab"));
-    SET_SHORTCUT(CloseTabAction, QKeySequence::Close);
+    CREATE_ACTION_ICON_SHORTCUT(CloseTabAction, "close", gt("close-tab"), QKeySequence::Close);
     mpa_actions[CloseTabAction]->setIconVisibleInMenu(false);
 
-    CREATE_ACTION(ReopenClosedTabAction, gt("reopen-closed-tab"));
-    SET_SHORTCUT(ReopenClosedTabAction, QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_T));
+    CREATE_ACTION_SHORTCUT(ReopenClosedTabAction, gt("reopen-closed-tab"), QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_T));
     HIDE_ACTION(ReopenClosedTabAction);
 
-    CREATE_ACTION(BrowseLibraryAction, gt("browse-library"));
-    SET_SHORTCUT(BrowseLibraryAction, QKeySequence(Qt::CTRL+Qt::Key_E));
+    CREATE_ACTION_SHORTCUT(BrowseLibraryAction, gt("browse-library"), QKeySequence(Qt::CTRL+Qt::Key_E));
     HIDE_ACTION(BrowseLibraryAction);
 
-    CREATE_ACTION_ICON(OpenFileAction, "open-file", gt("open-file"));
-    SET_SHORTCUT(OpenFileAction, QKeySequence::Open);
+    CREATE_ACTION_ICON_SHORTCUT(OpenFileAction, "open-file", gt("open-file"), QKeySequence::Open);
     connect(mpa_actions[OpenFileAction], &QAction::triggered,
             this, [=]() { openZimFile(); });
 
@@ -334,25 +370,22 @@ void KiwixApp::createAction()
 
     /* TODO See https://github.com/kiwix/kiwix-desktop/issues/77
     CREATE_ACTION(SavePageAsAction, tr("Save page as ..."));
-    SET_SHORTCUT(SavePageAsAction, QKeySequence::SaveAs);
+    // SET_SHORTCUT(SavePageAsAction, QKeySequence::SaveAs);
     HIDE_ACTION(SavePageAsAction);
     */
 
-    CREATE_ACTION(SearchArticleAction, gt("search-article"));
-    SET_SHORTCUT(SearchArticleAction, QKeySequence(Qt::CTRL+Qt::Key_L));
+    CREATE_ACTION_SHORTCUT(SearchArticleAction, gt("search-article"), QKeySequence(Qt::CTRL+Qt::Key_L));
     HIDE_ACTION(SearchArticleAction);
 
-    CREATE_ACTION(SearchLibraryAction, gt("search-in-library"));
-    SET_SHORTCUT(SearchLibraryAction, QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_R));
+    CREATE_ACTION_SHORTCUT(SearchLibraryAction, gt("search-in-library"), QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_R));
     HIDE_ACTION(SearchLibraryAction);
 
     CREATE_ACTION(FindInPageAction, gt("find-in-page"));
     mpa_actions[FindInPageAction]->setShortcuts({QKeySequence::Find, Qt::Key_F3});
     connect(mpa_actions[FindInPageAction], &QAction::triggered,
-            this, [=]() { mp_tabWidget->openFindInPageBar(); });
+            this, [=]() { getTabWidget()->openFindInPageBar(); });
 
-    CREATE_ACTION_ICON(ToggleFullscreenAction, "full-screen-enter", gt("set-fullscreen"));
-    SET_SHORTCUT(ToggleFullscreenAction, QKeySequence::FullScreen);
+    CREATE_ACTION_ICON_SHORTCUT(ToggleFullscreenAction, "full-screen-enter", gt("set-fullscreen"),  QKeySequence::FullScreen);
     connect(mpa_actions[ToggleFullscreenAction], &QAction::toggled,
             this, [=](bool checked) {
         auto action = mpa_actions[ToggleFullscreenAction];
@@ -362,31 +395,18 @@ void KiwixApp::createAction()
     });
     mpa_actions[ToggleFullscreenAction]->setCheckable(true);
 
-    CREATE_ACTION(ToggleTOCAction, gt("table-of-content"));
-    SET_SHORTCUT(ToggleTOCAction, QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_1));
+    CREATE_ACTION_SHORTCUT(ToggleTOCAction, gt("table-of-content"), QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_1));
     HIDE_ACTION(ToggleTOCAction);
 
-    CREATE_ACTION_ICON(ToggleReadingListAction, "reading-list" ,gt("reading-list"));
-    SET_SHORTCUT(ToggleReadingListAction, QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_2));
-    connect(mpa_actions[ToggleReadingListAction], &QAction::triggered,
-            this, [=]() { toggleSideBar(READINGLIST_BAR); });
-    connect(this, &KiwixApp::currentSideTypeChanged,
-            this, [=](SideBarType type) {
-        mpa_actions[ToggleReadingListAction]->setIcon(
-            QIcon((type == READINGLIST_BAR) ? ":/icons/reading-list-active.svg" : ":/icons/reading-list.svg"));
-    });
+    CREATE_ACTION_ONOFF_ICON_SHORTCUT(ToggleReadingListAction, "reading-list-active", "reading-list", gt("reading-list"), QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_2));
 
-    CREATE_ACTION(ZoomInAction, gt("zoom-in"));
-    SET_SHORTCUT(ZoomInAction, QKeySequence::ZoomIn);
+    CREATE_ACTION_SHORTCUTS(ZoomInAction, gt("zoom-in"), QList<QKeySequence>({QKeySequence::ZoomIn, QKeySequence(Qt::CTRL+Qt::Key_Equal)}));
 
-    CREATE_ACTION(ZoomOutAction, gt("zoom-out"));
-    SET_SHORTCUT(ZoomOutAction, QKeySequence::ZoomOut);
+    CREATE_ACTION_SHORTCUT(ZoomOutAction, gt("zoom-out"), QKeySequence::ZoomOut);
 
-    CREATE_ACTION(ZoomResetAction, gt("zoom-reset"));
-    SET_SHORTCUT(ZoomResetAction, QKeySequence(Qt::CTRL+Qt::Key_0));
+    CREATE_ACTION_SHORTCUT(ZoomResetAction, gt("zoom-reset"), QKeySequence(Qt::CTRL+Qt::Key_0));
 
-    CREATE_ACTION(HelpAction, gt("help"));
-    SET_SHORTCUT(HelpAction, QKeySequence::HelpContents);
+    CREATE_ACTION_SHORTCUT(HelpAction, gt("help"), QKeySequence::HelpContents);
     HIDE_ACTION(HelpAction);
 
     CREATE_ACTION(FeedbackAction, gt("feedback"));
@@ -400,23 +420,18 @@ void KiwixApp::createAction()
 
     CREATE_ACTION(AboutAction, gt("about-kiwix"));
 
-    CREATE_ACTION_ICON(SettingAction, "settings", gt("settings"));
-    SET_SHORTCUT(SettingAction, QKeySequence(Qt::Key_F12));
+    CREATE_ACTION_ICON_SHORTCUT(SettingAction, "settings", gt("settings"),  QKeySequence(Qt::Key_F12));
 
-    CREATE_ACTION_ICON(DonateAction, "donate", gt("donate-to-support-kiwix"));
-    SET_SHORTCUT(DonateAction, QKeySequence(Qt::CTRL+Qt::Key_D));
+    CREATE_ACTION_ICON_SHORTCUT(DonateAction, "donate", gt("donate-to-support-kiwix"),  QKeySequence(Qt::CTRL+Qt::Key_D));
 
-    CREATE_ACTION_ICON(ExitAction, "exit", gt("exit"));
-    SET_SHORTCUT(ExitAction, QKeySequence::Quit);
+    CREATE_ACTION_ICON_SHORTCUT(ExitAction, "exit", gt("exit"), QKeySequence::Quit);
 }
 
 void KiwixApp::postInit() {
-    connect(mp_tabWidget, &TabBar::webActionEnabledChanged,
-            mp_mainWindow->getTopWidget(), &TopWidget::handleWebActionEnabledChanged);
-    connect(mp_tabWidget, &TabBar::currentTitleChanged, this,
-            [=](const QString& title) { emit currentTitleChanged(title); });
-    connect(mp_tabWidget, &TabBar::libraryPageDisplayed, this, &KiwixApp::disableItemsOnLibraryPage);
+    connect(getTabWidget(), &TabBar::libraryPageDisplayed,
+            this, &KiwixApp::disableItemsOnLibraryPage);
     emit(m_library.booksChanged());
+    connect(&m_library, &Library::booksChanged, this, &KiwixApp::updateNameMapper);
     disableItemsOnLibraryPage(true);
 }
 
@@ -428,3 +443,17 @@ void KiwixApp::disableItemsOnLibraryPage(bool libraryDisplayed)
     KiwixApp::instance()->getAction(KiwixApp::ZoomOutAction)->setDisabled(libraryDisplayed);
     KiwixApp::instance()->getAction(KiwixApp::ZoomResetAction)->setDisabled(libraryDisplayed);
 }
+
+void KiwixApp::updateNameMapper()
+{
+  m_nameMapper.update();
+}
+
+void KiwixApp::printVersions(std::ostream& out) {
+  out << version.toStdString() << std::endl;
+  out << "+ libqt (compile time) " << QT_VERSION_STR << std::endl;
+  out << "+ libqt (run time) " << qVersion() << std::endl << std::endl;
+  kiwix::printVersions(out);
+  out << std::endl;
+  zim::printVersions(out);
+}
