@@ -11,10 +11,8 @@
 
 UrlSchemeHandler::UrlSchemeHandler()
 {
-
 }
 
-
 void
 UrlSchemeHandler::handleContentRequest(QWebEngineUrlRequestJob *request)
 {
@@ -30,35 +28,24 @@ UrlSchemeHandler::handleContentRequest(QWebEngineUrlRequestJob *request)
         request->fail(QWebEngineUrlRequestJob::UrlNotFound);
         return;
     }
-    kiwix::Entry entry;
     try {
-        entry = reader->getEntryFromPath(url);
-    } catch (kiwix::NoEntry&) {
-        url = "A/" + url;
-        try {
-            entry = reader->getEntryFromPath(url);
-        } catch (kiwix::NoEntry&) {
-            request->fail(QWebEngineUrlRequestJob::UrlNotFound);
-            return;
-        }
-    }
-    if (entry.isRedirect()) {
-        try {
+        kiwix::Entry entry = reader->getEntryFromPath(url);
+        if (entry.isRedirect()) {
             entry = entry.getFinalEntry();
-        } catch (kiwix::NoEntry&) {
-            request->fail(QWebEngineUrlRequestJob::UrlNotFound);
+            auto path = QString("/") + QString::fromStdString(entry.getPath());
+            qurl.setPath(path);
+            request->redirect(qurl);
             return;
         }
-        auto path = QString("/") + QString::fromStdString(entry.getPath());
-        qurl.setPath(path);
-        request->redirect(qurl);
-        return;
+
+        BlobBuffer* buffer = new BlobBuffer(entry.getBlob());
+        auto mimeType = QByteArray::fromStdString(entry.getMimetype());
+        mimeType = mimeType.split(';')[0];
+        connect(request, &QObject::destroyed, buffer, &QObject::deleteLater);
+        request->reply(mimeType, buffer);
+    } catch (kiwix::NoEntry&) {
+      request->fail(QWebEngineUrlRequestJob::UrlNotFound);
     }
-    BlobBuffer* buffer = new BlobBuffer(entry.getBlob());
-    auto mimeType = QByteArray::fromStdString(entry.getMimetype());
-    mimeType = mimeType.split(';')[0];
-    connect(request, &QObject::destroyed, buffer, &QObject::deleteLater);
-    request->reply(mimeType, buffer);
 }
 
 void
@@ -88,8 +75,8 @@ UrlSchemeHandler::handleMetaRequest(QWebEngineUrlRequestJob* request)
 
 
 class IdNameMapper : public kiwix::NameMapper {
-  std::string getNameForId(const std::string& id) { return id + ".zim"; }
-  std::string getIdForName(const std::string& id) { return id.substr(0, id.size()-4); }
+  std::string getNameForId(const std::string& id) const { return id + ".zim"; }
+  std::string getIdForName(const std::string& id) const { return id.substr(0, id.size()-4); }
 };
 
 
@@ -111,10 +98,12 @@ UrlSchemeHandler::handleSearchRequest(QWebEngineUrlRequestJob* request)
     int temp = query.queryItemValue("start").toInt(&ok);
     if (ok)
       start = temp;
-    int end = 25;
-    temp = query.queryItemValue("end").toInt(&ok);
+    int pageLength = 25;
+    temp = query.queryItemValue("pageLength").toInt(&ok);
     if (ok)
-      end = temp;
+      pageLength = temp;
+
+    auto end = start + pageLength;
 
     auto searcher = app->getLibrary()->getSearcher(bookId);
     searcher->search(searchQuery, start, end);
@@ -125,6 +114,7 @@ UrlSchemeHandler::handleSearchRequest(QWebEngineUrlRequestJob* request)
     renderer.setSearchContent(bookId.toStdString());
     renderer.setProtocolPrefix("zim://");
     renderer.setSearchProtocolPrefix("zim://" + host.toStdString() + "/?");
+    renderer.setPageLength(pageLength);
     auto content = renderer.getHtml();
     QBuffer *buffer = new QBuffer;
     buffer->setData(content.data(), content.size());
